# SPDX-FileCopyrightText: 2023-present João Antonio Ferreira <joao.parana@gmail.com>
#
# SPDX-License-Identifier: MIT
from .__about__ import __version__
# from abc import ABC, abstractmethod
#
# class ITimeSerie(ABC):
#    pass